/* This file is part of epsilon, a functional language implementation

epsilon v0.01
Copyright (C) Luca Saiu, 2002

This is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This software is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public
License along with this file; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#ifndef TYPES_H_
#define TYPES_H_

#include "term.h"

#include <stdio.h>

typedef term_t type_t;

extern type_t type_error; /* a special type */
extern type_t integer_type,float_type,boolean_type,string_type;

void initialize_types_table();
void destroy_types_table();

type_t unify_types(type_t a,type_t b);
int do_types_match(type_t a,type_t b);

type_t new_list_type(type_t x);
type_t new_pair_type(type_t x,type_t y);
type_t new_function_type(type_t x,type_t y);
type_t new_array_type(type_t x);
type_t new_promise_type(type_t x);
type_t new_postfix_operator_type(type_t from, type_t to);
type_t new_infix_operator_type(type_t from_1, type_t from_2, type_t to);

int is_a_list_type(type_t a);
int is_a_pair_type(type_t a);
int is_a_function_type(type_t a);
int is_an_array_type(type_t a);
int is_a_promise_type(type_t a);
int is_a_postfix_operator_type(type_t a);
int is_an_infix_operator_type(type_t a);

int arity_of_tuple_type(type_t a); /* this assumes a is a tuple type */
type_t element_of_tuple_type(type_t x,int n); /* n starts at 1. */

type_t generic_type(char* name);
type_t new_generic_type();
int is_a_generic_type(type_t a);

type_t element_type(type_t t,int n);

int are_types_equal(type_t m,type_t n);
int are_types_equal_excluding_variables(type_t m,type_t n);

void output_type(FILE* f,type_t t);

#endif /* #ifndef TYPES_H_ */

